﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Ants
{
    public partial class Form1 : Form
    {
        public float fScale = 0.2F;
        World fworld;
        public Timer fMainLoopTimer;
        int timeCount;
        int fRunLength;
        int fRepNum;
        Output fOutput;
        bool withPheromone =true;
        bool withAnts = true;
        bool withLabels = false;
        bool withFood = true;
        Graphics surface;

        //Constructor
        public Form1(World aworld, Output aOutput, int aRepNum, int aRunLength)
        {
            InitializeComponent();

            fworld = aworld;
            fRunLength = aRunLength;
            fOutput = aOutput;
            fRepNum = aRepNum;
            surface = this.fPictureBoxDisplay.CreateGraphics();
            this.SetStyle(
                ControlStyles.UserPaint 
                |ControlStyles.AllPaintingInWmPaint 
                |ControlStyles.OptimizedDoubleBuffer, true);
            surface.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            timeCount = 0;
            fMainLoopTimer = new Timer();
            fMainLoopTimer.Interval = 1;
            fMainLoopTimer.Tick += new EventHandler(OnMainLoopTimerTick);
            fMainLoopTimer.Start();
        }

        //Runs at start
        void Form1_Shown(object sender, EventArgs e)
        {
            panel1.AutoScrollPosition = new Point((int)(World.fSize.Width * fScale / 2) - (panel1.Width / 2), (int)(World.fSize.Height * fScale / 2) - (panel1.Height / 2));
        }

        void OnMainLoopTimerTick(object sender, EventArgs e)
        {

            if (!panel1.IsDisposed && !fPictureBoxDisplay.IsDisposed)
            {
                if (!panel1.Size.IsEmpty && (panel1.Size.Height != 0) && (panel1.Size.Width != 0))
                {
                    // Not minimised, draw the stuff
                    fPictureBox.Size = new Size((int)(World.fSize.Width * fScale), (int)(World.fSize.Height * fScale));
                    Program.RunStep(fworld, fOutput, timeCount, fRepNum, surface);
                    Bitmap lBitmap = new Bitmap(panel1.Size.Width, panel1.Size.Height);
                    Graphics lGraphics = Graphics.FromImage(lBitmap);
                    Point lOffset = fPictureBox.PointToClient(panel1.PointToScreen(new Point()));

                    //Update everything and draw
                    fPictureBoxDisplay.Image = lBitmap;
                    fPictureBoxDisplay.Left = 0;
                    fPictureBoxDisplay.Top = 0;

                    DrawPicture(fworld, surface, fOutput);
                }
            }
            else
            {
                Program.RunStep(fworld, fOutput, timeCount, fRepNum, null);
            }
            
            if (timeCount > fRunLength)
            {
                fMainLoopTimer.Stop();
                this.DialogResult = DialogResult.OK;
                this.Close();
            }
            timeCount++;
        }

        public void DrawPicture(World aworld, Graphics aSurface, Output aOutput )
        {
            
            fTimeBox.Text = aworld.Age.ToString("0");
            foreach (Colony acolony in aworld.Colonies)
            {
                aOutput.ColonyDraw(aSurface, acolony, withPheromone, withAnts, withLabels);
            }
            if (withFood)
            {
                Output.FoodDraw(aSurface, aworld);
            }
        }

        private void OnResize(object sender, EventArgs e)
        {
            fPictureBoxDisplay.Bounds = panel1.Bounds;
        }

        private void fScaleBar_Scroll(object sender, ScrollEventArgs e)
        {
            fScale = (float)e.NewValue / 10.0F;
        }

        private void stop_button_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Abort;
            this.Close();
        }
   }
}
