using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Ants
{

    public class Nest: Food

    {
        public List<int> NestWaitingTimes = new List<int>();
        public List<Food> FoundFoods = new List<Food>();
        public List<int> FoundFoodTimes = new List<int>();
        public List<float> FoundFoodAmounts = new List<float>();
        public int StealCount;
        public float StealAmount;
        public int FirstFoundFood;
        public int NumberForOutput;

        public World World
        {
            get { return fWorld; }
        }

        public Colony Colony
        {
            get { return fColony; }
        }
        
        public double Random
        {
            get { return fWorld.Random; }
        }

        public float Normal (float mean, float stdev)
        {
            return fWorld.Normal(mean, stdev);
        }

        public List<Ant> fAnts = new List<Ant>();

        public List<Ant> Leaders
        {
            set
            {
                fLeadersInNest = value;
            }
            get
            {
                return fLeadersInNest;
            }
        }

        public List<Ant> WaitingAnts
        {
            set
            {
                fAntsInNest = value;
            }
            get { return fAntsInNest; }
        }

        public Dictionary<Nest, int> ArrivalsFromNests
        {
            get { return fArrivalsFromNests; }
        }

        List<Ant>fLeadersInNest = new List<Ant>();
        List<Ant> fAntsInNest = new List<Ant>();
        World fWorld;
        Colony fColony;
        int fForagingAnts;
        int fTransporterAnts;

        Dictionary<int, Dictionary<int, List<Pheromone>>> fFoodPheromoneMap = new Dictionary<int, Dictionary<int, List<Pheromone>>>();
        Dictionary<int, Dictionary<int, List<Pheromone>>> fHomePheromoneMap = new Dictionary<int, Dictionary<int, List<Pheromone>>>();
        Dictionary<Nest, int> fArrivalsFromNests = new Dictionary<Nest, int>();

        //Constructor
        public Nest(int aForagingAnts, int aTransporterAnts, World aWorld, PointF aPosition, Colony aColony, int aNumberForOutput, float aAntSpeed, float aAntSpeedStDev)
            : base(aPosition, 0)
        {
            fWorld = aWorld;
            fForagingAnts = aForagingAnts;
            fTransporterAnts = aTransporterAnts;
            StealCount = 0;
            StealAmount = 0;
            FirstFoundFood = fWorld.Age;
            fColony = aColony;
            NumberForOutput = aNumberForOutput;

            for (int i = 0; i < fForagingAnts; ++i)
            {
                fAnts.Add(new Ant(this, aWorld.RouteFidelity, Ant.AntType.Forager, aAntSpeed, aAntSpeedStDev));
            }
            
            for (int i = 0; i < fTransporterAnts; ++i)
            {
                fAnts.Add(new Ant(this, aWorld.RouteFidelity, Ant.AntType.Transporter, aAntSpeed, aAntSpeedStDev));
            }
        }

        //Shuffle then loop all ants in this nest
        public void Step(Graphics graphics)
        {
            //Shuffle ants
            World.FYShuffleList(fAnts);
            // Update ants
            foreach (Ant lAnt in fAnts)
            {
                lAnt.Step();
            }
            //deplete nest food
            this.Amount -= this.fAnts.Count * Ant.RestingMR * this.fAnts[1].Weight;
        }

        //Draws single nest and all ants within that nest if specified
        public void Draw(Graphics aGraphics)
        {
            Pen lPen = new Pen(Color.Goldenrod);
            aGraphics.DrawEllipse(lPen, (Position.X - Nest.DetectRange), (Position.Y - Nest.DetectRange), Nest.DetectRange * 2, Nest.DetectRange * 2);
            aGraphics.FillEllipse(lPen.Brush, (Position.X - Nest.fRadius), (Position.Y - Nest.fRadius), Nest.fRadius * 2, Nest.fRadius * 2);
        }
    }
}
